<?php
require_once('../functions/mailbox_display.php');
require_once('../plugins/bayesspam/config.php');

function bayesspam_filter($args) {
	if (!$GLOBALS['bayesspam_filtering_enabled'])
		return 0;

	// Detect if we have already connected to IMAP or not.
	// Also check if we are forced to use a separate IMAP connection
	if ((!isset($GLOBALS['imap_stream']) && !isset($GLOBALS['imapConnection'])) || (isset($GLOBALS['UseSeparateImapConnection']) && $GLOBALS['UseSeparateImapConnection'])) {
		$stream = sqimap_login($_SESSION['username'], $_COOKIE['key'], $GLOBALS['imapServerAddress'], $GLOBALS['imapPort'], 10);
		$previously_connected = false;
	} elseif (isset($GLOBALS['imapConnection'])) {
		$stream = $GLOBALS['imapConnection'];
		$previously_connected = true;
	} else {
		$previously_connected = true;
		$stream = $GLOBALS['imap_stream'];
	}

	if($args[0] == 'left_main_before') {
		bayesspam_filters($stream, 0);
	} else {
		bayesspam_filters($stream, 1);
	}

	if (!$previously_connected) {
		sqimap_logout($stream);
	}
}

function bayesspam_filters(&$imap_stream, $use_mailbox=0) {
	$version_array = split('\.', $GLOBALS['version']);

	$do_autolearn_spam = 0;
	$do_autolearn_nonspam = 0;
	$new_spam_occurences = array();
	$new_nonspam_occurences = array();

	$run_folders = array();
	$boxes = sqimap_mailbox_list($imap_stream);
	$sid = sqimap_session_id();

	foreach ($boxes as $box) {
		if ($GLOBALS['bayesspam_inboxonly'] != 1 || ($GLOBALS['bayesspam_inboxonly'] == 1 && preg_match("/^INBOX$/", $box['unformatted-dm']))) {
			if (($use_mailbox == 1 && $box['unformatted-dm'] == $GLOBALS['mailbox'] && !in_array($box['unformatted-dm'],$GLOBALS['bayesspam_ignore_folders'])) || ($use_mailbox == 0 && !in_array($box['unformatted-dm'], $GLOBALS['bayesspam_ignore_folders']))) {
				if ( (array_search('noselect', $box['flags']) === FALSE || array_search('noselect', $box['flags']) === NULL) && !preg_match("/^INBOX\.Sent/", $box['unformatted-dm']) && sqimap_unseen_messages($imap_stream, $box['unformatted-dm']) > 0) {
					$run_folders[] = $box['unformatted-dm'];
				}
			}
		}
	}

	foreach ($run_folders as $box) {
		$spam_messages = array();
		$qty = sqimap_get_num_messages ($imap_stream, $box);
		$id = range(1, $qty);

		$mbxresponse = sqimap_mailbox_select($imap_stream, $box);

		$msgs_list = sqimap_get_small_header_list($imap_stream, $id, 0);
		$flag_list = sqimap_get_flags_list($imap_stream, $id);

		$which = 0;
		$messages = array();
		if (sizeof($msgs_list)){
			foreach ($msgs_list as $hdr) {
				if (array_search('Seen', $flag_list[$which]) === NULL || array_search('Seen', $flag_list[$which]) === FALSE) {
					$messages[] = $id[$which];
				}
				$which++;
			 }
		}

		if ($GLOBALS['bayesspam_autolearn'] && !$GLOBALS['bayesspam_cached_occurences'] && count($messages)) {
			$GLOBALS['bayesspam_spam_occurences'] = bayesspam_get_occurences('spam');
			$GLOBALS['bayesspam_nonspam_occurences'] = bayesspam_get_occurences('nonspam');
			$GLOBALS['bayesspam_cached_occurences'] = 1;
		}

		foreach ($messages as $passed_id) {
			if ($GLOBALS['bayesspam_autolearn'] && !bayesspam_check_messageid()) {
				$is_spam = bayesspam_get_probability($imap_stream, $passed_id, 1);
			} else {
				$is_spam = bayesspam_get_probability($imap_stream, $passed_id);
			}

			if ($is_spam>.9) {
				$spam_messages[] = $passed_id;
			}
			if ($GLOBALS['bayesspam_autolearn'] && !bayesspam_check_messageid()) {
				$type = '';
				if ($is_spam > .9 ) {
					$type = 'spam';
				} else {
					$type = 'nonspam';
				}
				$added_spam = 0;
				$added_nonspam = 0;
				foreach ($GLOBALS['bayesspam_tokens'] as $token) {
					if (strlen($token) > 0) {
						if ($is_spam > .9) {
							$added_spam = 1;
							@$new_spam_occurences['token'.$token]++;
						} else {
							$added_nonspam = 1;
							@$new_nonspam_occurences['token'.$token] += 2;
						}
					}
				}
				if ($added_nonspam) {
					$do_autolearn_nonspam++;
				}
				if ($added_spam) {
					$do_autolearn_spam++;
				}
				bayesspam_add_messageid($type);
			}	
		}

		if ($spam_messages) {
			$message_str = sqimap_message_list_squisher($spam_messages);
			if ($GLOBALS['bayesspam_delete']) {
				sqimap_run_command ($imap_stream, 'STORE '.$message_str.' +FLAGS (\Deleted)', true, $response, $message);
			} elseif (sqimap_mailbox_exists($imap_stream, $GLOBALS['bayesspam_folder'])) {
				sqimap_run_command ($imap_stream, 'COPY '.$message_str.' "'.$GLOBALS['bayesspam_folder'].'"', true, $response, $message);
				sqimap_run_command ($imap_stream, 'STORE '.$message_str.' +FLAGS (\Deleted)', true, $response, $message);
			}
	
	    	sqimap_mailbox_expunge($imap_stream, $box);
		}
	}

	if ($GLOBALS['bayesspam_autolearn'] && ($do_autolearn_spam || $do_autolearn_nonspam)) {
		if ($do_autolearn_spam) {
			bayesspam_do_learn('spam',$spam_occurences,$new_spam_occurences,$do_autolearn_spam,1);
		}
		
		if ($do_autolearn_nonspam) {
			bayesspam_do_learn('nonspam',$nonspam_occurences,$new_nonspam_occurences,$do_autolearn_nonspam,1);
		}
	}
}
?>
